/*
 *	Tykkipeli 2
 *	Control panel stuff.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>

#include "defines.h"
#include "tp2.h"
#include "game_screen.h"
#include "players.h"
#include "weapons.h"
#include "shields.h"
#include "guns.h"
#include "change_item.h"
#include "info_windows.h"
#include "terrain.h"
#include "control_panel_defines.h"
#include "shield_defines.h"

#include "hr_colors.h"
#include "hr_bitmaps.h"
#include "hr_buttons.h"
#include "hr_mouse.h"
#include "hr_bobs.h"
#include "hr_zones.h"
#include "hr_fonts.h"
#include "hr_defines.h"
#include "hr_general.h"
#include "hr_input_handler.h"
#include "hr_keyboard.h"
#include "hr_graphics.h"

#define MAX_CONTROLBUTTONS 13
struct hr_button **ctrlbutton;
struct hr_zone *game_area;


/* Modifies player p's angle by amount degrees.*/
void add_angle(struct player *p,int amount)
{
	(p->angle)+=amount;
	if((p->angle)<0)
	  (p->angle)=0;
	if((p->angle)>180)
	  (p->angle)=180;
}


/* Modifies player p's power.*/
void add_power(struct player *p,int amount)
{
	(p->power)+=amount;
	if((p->power)<MIN_POWER)
	  (p->power)=MIN_POWER;
	if((p->power)>MAX_POWER)
	  (p->power)=MAX_POWER;
}


/* Updates ALL panel bars. Temporarily changes clippingwindow.*/
void panel_bars()
{
	int z,l,h;
	int cx1,cy1,cx2,cy2;
	struct player *p;
	struct shield_list *sl;
	/* Damage bars */
#define BARW (S_WIDTH-524)
	/* Calc the amount of players for calcing h...*/
	z=0;p=firstplayer;
	while(p!=NULL){
		p=p->next;
		++z;
	}
	h=62/z;
	cx1=__clipx1;
	cy1=__clipy1;
	cx2=__clipx2;
	cy2=__clipy2;
	gl_setclippingwindow(522,PANEL_Y+16,MAX_X-1,S_HEIGHT-2);
	gl_fillbox(522,PANEL_Y+15,BARW,64,0);
	l=0;
	p=firstplayer;
	while(p!=NULL) {
		/* Damage bar */
		if((p->damage)>0)
		  gl_fillbox(522,(PANEL_Y+16)+l*h,
						 ((p->damage)*BARW)/MAX_DAMAGE,
						 h/2-1,p->britecolor);
		/* Shield bar */
		if((p->active_shield->index)!=CHARS2INT("None")){
			sl=p->shields;
			while((sl->type)!=(p->active_shield))
			  sl=sl->next;
			if((sl->amount)>0)
			  gl_fillbox(522,(PANEL_Y+16)+l*h+h/2,
							 ((sl->amount)*BARW)/(sl->type->efficiency),
							 h/2-1,p->darkcolor);
		}
		++l;
		p=p->next;
	}
	gl_setclippingwindow(cx1,cy1,cx2,cy2);
}


/* Creates the control panel. The control panel area is ALWAYS
 * 80 pixels tall. */
void create_control_panel()
{
	struct hr_button *b;
	
	ctrlbutton=calloc(MAX_CONTROLBUTTONS,sizeof(struct hr_button *));
	gl_setclippingwindow(0,0,MAX_X,S_HEIGHT-1);
	
	/* Game area */
	game_area=hr_create_zone(0,0,MAX_X,MAX_Y,ZONE_ENABLED);
	/* First build the borders -------------------------------------*/
	/* Windmeter border */
	hr_bevelbox(S_WIDTH-WINDMETER_WIDTH,PANEL_Y+1,MAX_X,PANEL_Y+12,
					hr.def_colors,BUTTON_PRESSED);
	/* Player info */
	hr_bevelbox(0,PANEL_Y+14,158,PANEL_Y+48,hr.def_colors,BUTTON_PRESSED);
   hr_write(2,PANEL_Y+37,1,0,"$",hr.font8x8,STRING_START,STRING_END,NONE);
	hr_write(73,PANEL_Y+37,1,0,"Ratio",hr.font8x8,STRING_START,STRING_END,NONE);
	
	/* Bottomleft bevelboxes */
	hr_bevelbox(0,PANEL_Y+50,69,S_HEIGHT-1,hr.def_colors,BUTTON_PRESSED);
	ctrlbutton[11]=hr_create_button(2,PANEL_Y+52,67,PANEL_Y+63,
												hr.font8x8,hr.def_colors,
												"~uC~uolors",NULL,'c',
												BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	ctrlbutton[12]=hr_create_button(2,PANEL_Y+65,67,S_HEIGHT-3,
												hr.font8x8,hr.def_colors,
												"~uS~uettings",NULL,'s',
												BUTTON_NO_TAB_CYCLE);
	hr_bevelbox(71,PANEL_Y+50,158,S_HEIGHT-1,hr.def_colors,BUTTON_PRESSED);
	/* Turn and round.*/
	hr_write(73,PANEL_Y+54,1,0,"Turn",hr.font8x8,STRING_START,STRING_END,NONE);
	hr_write(73,PANEL_Y+66,1,0,"Round",hr.font8x8,STRING_START,STRING_END,NONE);
	/* Main control area*/
	hr_bevelbox(160,PANEL_Y+14,519,S_HEIGHT-1,hr.def_colors,BUTTON_PRESSED);
	hr_write(165,PANEL_Y+19,1,0,"Angle",hr.font8x8,
				STRING_START,STRING_END,NONE);
   hr_write(165,PANEL_Y+35,1,0,"Power",hr.font8x8,
				STRING_START,STRING_END,NONE);
   hr_write(165,PANEL_Y+51,1,0,"Weapon",hr.font8x8,
				STRING_START,STRING_END,NONE);
   hr_write(165,PANEL_Y+66,1,0,"Shield",hr.font8x8,
				STRING_START,STRING_END,NONE);
	/* Underlines.*/
   hr_write(165,PANEL_Y+52,1,0,"_",hr.font8x8,STRING_START,STRING_END,NONE);
   hr_write(165,PANEL_Y+67,1,0,"_",hr.font8x8,STRING_START,STRING_END,NONE);
	/* The rest is for damage/shield bars */
	hr_bevelbox(521,PANEL_Y+14,MAX_X,S_HEIGHT-1,hr.def_colors,BUTTON_PRESSED);
	/* Then create the needed buttons ------------------------------*/
	strcpy(buffer," ");
	/* Weapon/shield*/
	ctrlbutton[0]=hr_create_button(218,PANEL_Y+49,450,PANEL_Y+61,
											 hr.font8x8,hr.def_colors,
											 buffer,"Select new ~uw~ueapon",'w',
											 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	ctrlbutton[1]=hr_create_button(218,PANEL_Y+64,450,PANEL_Y+76,
											 hr.font8x8,hr.def_colors,
											 buffer,"Select new ~us~uhield",'s',
											 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	/* Angle */
	ctrlbutton[2]=hr_create_button(218,PANEL_Y+16,231,PANEL_Y+29,
											 hr.font8x8,hr.def_colors,
											 "<",NULL,'<',
											 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE|
											 BUTTON_NO_DELAY);
   ctrlbutton[3]=hr_create_button(233,PANEL_Y+16,263,PANEL_Y+29,
											 hr.font8x8,hr.def_colors,
											 buffer,NULL,0,
											 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
   ctrlbutton[4]=hr_create_button(265,PANEL_Y+16,278,PANEL_Y+29,
											 hr.font8x8,hr.def_colors,
											 ">",NULL,'>',
											 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE|
											 BUTTON_NO_DELAY);
	/* QuickoAngle(tm)*/
	b=ctrlbutton[8]=hr_create_button(280,PANEL_Y+16,451,PANEL_Y+29,
												hr.font8x8,hr.def_colors,
												NULL,NULL,0,
												BUTTON_HSLIDER|BUTTON_ENABLED
												 |BUTTON_PRESSED|BUTTON_NO_TAB_CYCLE);
	b->extras=hr_create_slider(BUTTON_HSLIDER|SLIDER_XNEG|
										SLIDER_GET_XVALINT|SLIDER_SET_XVALINT,
										181,10,30,180,&(active_player->angle),
										0,0,0,0,NULL);
	/* Power */
	ctrlbutton[5]=hr_create_button(218,PANEL_Y+32,231,PANEL_Y+45,
											 hr.font8x8,hr.def_colors,
											 "-",NULL,'-',
											 BUTTON_ENABLED
											 |BUTTON_NO_TAB_CYCLE|BUTTON_NO_DELAY);
	ctrlbutton[6]=hr_create_button(233,PANEL_Y+32,263,PANEL_Y+45,
											 hr.font8x8,hr.def_colors,
											 buffer,NULL,0,
											 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	ctrlbutton[7]=hr_create_button(265,PANEL_Y+32,278,PANEL_Y+45,
											 hr.font8x8,hr.def_colors,
											 "+",NULL,'+',
											 BUTTON_ENABLED
											 |BUTTON_NO_TAB_CYCLE|BUTTON_NO_DELAY);
	/* QuickoPower(tm)*/
	b=ctrlbutton[9]=hr_create_button(280,PANEL_Y+32,451,PANEL_Y+45,
												hr.font8x8,hr.def_colors,
												NULL,NULL,0,
												BUTTON_HSLIDER|BUTTON_ENABLED
												|BUTTON_PRESSED|BUTTON_NO_TAB_CYCLE);
	b->extras=hr_create_slider(BUTTON_HSLIDER|SLIDER_GET_XVALINT|SLIDER_SET_XVALINT,
										MAX_POWER-MIN_POWER+1,37,90,MIN_POWER,
										&(active_player->power),
										0,0,0,0,NULL);
	/* Fire */
	ctrlbutton[10]=hr.default_button=b=
	  hr_create_button(453,PANEL_Y+17,516,PANEL_Y+45,
							 hr.font8x8,hr.def_colors,
							 "~uF~uire","~uF~uire!",'f',BUTTON_ENABLED);
}


/* Updates the control panel buttons and meters. */
void update_control_panel(struct player *p,int flags)
{
	struct weapon_list *wl;
	struct shield_list *sl;
	int xx=0,yy=0,bflags;

	if(flags & PANEL_NAME) {		
		/* Name */
		/*kludge*/
		hr_set_color_index(temp_colors,COLOR_FOREGROUND,p->britecolor);
		hr_textbox(2,PANEL_Y+16,156,PANEL_Y+33,
					  hr.font8x8,temp_colors,
					  p->name,STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
		/* $$$$ */
		sprintf(buffer,"%d",p->wealth);
		hr_textbox(12,PANEL_Y+35,69,PANEL_Y+46,
					  hr.font8x8,hr.def_colors,
					  buffer,STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
		sprintf(buffer,"%d",magic_ratio(p->deaths,p->kills,p->suicides));
		hr_textbox(115,PANEL_Y+35,156,PANEL_Y+46,
					  hr.font8x8,hr.def_colors,
					  buffer,STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
		/* Round. This needs to be updated only once per round,
		 * so it's located here :-) */
		sprintf(buffer,"%d/%d",round+1,round_n);
		hr_textbox(115,PANEL_Y+65,156,PANEL_Y+76,
					  hr.font8x8,hr.def_colors,
					  buffer,STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
	}

	if(flags & PANEL_WIND) {
		int w;
		/* Clear windmeter */
		gl_fillbox(S_WIDTH-WINDMETER_WIDTH+1,PANEL_Y+2,
					  WINDMETER_WIDTH-2,10,0);
		if(windmode>0) {
			/* Bar */
			w=MAX(1,((abs(wind)*(WINDMETER_WIDTH-2))/(MAX_WIND*windmode)/2));
			gl_fillbox(MAX_X-WINDMETER_WIDTH/2-((wind<0)?(w):(0)),
						  PANEL_Y+3,w,7,1);
		}
		/* Blue middle line */
		gl_line(MAX_X-WINDMETER_WIDTH/2,PANEL_Y+2,
				  MAX_X-WINDMETER_WIDTH/2,PANEL_Y+11,3);
	}
	
	if(flags & PANEL_TURN) {
		/* Turn */
		sprintf(buffer,"%d",turn);
		hr_textbox(115,PANEL_Y+52,156,PANEL_Y+63,
					  hr.font8x8,
					  hr.def_colors,buffer,STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
		/* Recolor/Settings-buttons.
		 * This is a good place to update them, since it
		 * only needs to be done once a turn.*/
		bflags=BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE;
		hr_update_button(ctrlbutton[11],UPDATE_FLAGS,&bflags);
		bflags=BUTTON_NO_TAB_CYCLE;
		hr_update_button(ctrlbutton[12],UPDATE_FLAGS,&bflags);
	}

	if(flags & PANEL_ANGLE) {
		/* QuickoAngle(tm)*/
		bflags=(BUTTON_ENABLED|BUTTON_PRESSED|BUTTON_HSLIDER|BUTTON_NO_TAB_CYCLE);
		hr_update_button(ctrlbutton[8],UPDATE_FLAGS,&bflags);
		/* Angle .*/
		/* Gun + AimAid(tm) */
		gl_setclippingwindow(0,0,MAX_X,MAX_Y);
		if(game_flags & AIM_AID) {
			xx=(p->x)+((pcos[p->angle]*BARREL_LEN*6)>>SCALE);
			yy=(p->y)-BARREL_OFFSET-((psin[p->angle]*BARREL_LEN*6)>>SCALE);
			hr_update_bob(crosshairs,0,0,NULL,BOB_REMOVE);
		}
		update_gun(p,BOB_SMART,NONE);
		if(game_flags & AIM_AID)
		  hr_update_bob(crosshairs,xx,yy,NULL,BOB_INIT);
		gl_setclippingwindow(0,0,MAX_X,S_HEIGHT-1);
		/* control panel stuff.*/
		bflags=(((p->angle)<180)?
				  (BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE|BUTTON_NO_DELAY;
		hr_update_button(ctrlbutton[2],UPDATE_FLAGS,&bflags);
		sprintf(buffer,"%d",p->angle);
		bflags=(((flags & PANEL_FREELY)?(BUTTON_PRESSED):(NONE))|BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
		hr_update_button(ctrlbutton[3],UPDATE_FLAGS,&bflags);
		hr_update_button(ctrlbutton[3],UPDATE_TEXT,buffer);
		bflags=(((p->angle)>0)?
				  (BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE|BUTTON_NO_DELAY;
		hr_update_button(ctrlbutton[4],UPDATE_FLAGS,&bflags);
	}
	
	if(flags & PANEL_POWER) {
		/* QuickPower(tm)*/
		bflags=(BUTTON_ENABLED|BUTTON_PRESSED|BUTTON_HSLIDER|BUTTON_NO_TAB_CYCLE);
		hr_update_button(ctrlbutton[9],UPDATE_FLAGS,&bflags);
		/* Power . Buttons = 5,6,7*/
		bflags=((((p->power)>MIN_POWER)?(BUTTON_ENABLED):(NONE))
				  |BUTTON_NO_TAB_CYCLE|BUTTON_NO_DELAY);
		hr_update_button(ctrlbutton[5],UPDATE_FLAGS,&bflags);
		sprintf(buffer,"%d",p->power);
		bflags=(((flags & PANEL_FREELY)?(BUTTON_PRESSED):(NONE))
				  |BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
		hr_update_button(ctrlbutton[6],UPDATE_FLAGS,&bflags);
		hr_update_button(ctrlbutton[6],UPDATE_TEXT,buffer);
		bflags=((((p->power)<MAX_POWER)?(BUTTON_ENABLED):(NONE))
				  |BUTTON_NO_TAB_CYCLE|BUTTON_NO_DELAY);
		hr_update_button(ctrlbutton[7],UPDATE_FLAGS,&bflags);
	}
	
	if(flags & PANEL_FIRE) {		
		/* FIRE! Button 8 */
		bflags=BUTTON_ENABLED;
		hr_update_button(ctrlbutton[10],UPDATE_FLAGS,&bflags);
	}
	
	if(flags & PANEL_WEAPON) {
		/* Active Weapon - Button 0 */
		bflags=(BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
		hr_update_button(ctrlbutton[0],UPDATE_FLAGS,&bflags);
		hr_update_button(ctrlbutton[0],UPDATE_TEXT,p->active_weapon->name);
		if(p->active_weapon->default_amount<0)			
		  sprintf(buffer,"Inf.");
		else{
			wl=p->weapons;
			while((wl->type)!=(p->active_weapon))
			  wl=wl->next;
			sprintf(buffer,"%d",wl->amount);
		}
		hr_textbox(452,PANEL_Y+49,517,PANEL_Y+61,
					  hr.font8x8,
					  hr.def_colors,buffer,STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
	}
	
	if(flags & PANEL_SHIELD) {		
		/* Active Shield - Button 1 */
		bflags=(BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
		hr_update_button(ctrlbutton[1],UPDATE_FLAGS,&bflags);
		hr_update_button(ctrlbutton[1],UPDATE_TEXT,p->active_shield->name);
		if((p->active_shield->index)==CHARS2INT("None"))
		  sprintf(buffer,"-");
		else{
			sl=p->shields;
			while((sl->type)!=(p->active_shield))
			  sl=sl->next;
			sprintf(buffer,"%d%%",
					  ((sl->amount)*100)/(sl->type->efficiency));
		}
		hr_textbox(452,PANEL_Y+64,517,PANEL_Y+76,
					  hr.font8x8,
					  hr.def_colors,buffer,STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
	}
	
	if(flags & PANEL_BARS)
	  panel_bars();
}


/* Prints a message to the control panel message area.
 * Temporarily changes clipping window.*/
void message(int color,unsigned char *msg)
{
	int cx1,cy1,cx2,cy2;
	if(msg!=NULL) {
		cx1=__clipx1;
		cy1=__clipy1;
		cx2=__clipx2;
		cy2=__clipy2;
		gl_setclippingwindow(0,PANEL_Y+1,MAX_X-WINDMETER_WIDTH,PANEL_Y+12);
		hr_set_color_index(temp_colors,COLOR_FOREGROUND,color);
		hr_textbox(0,PANEL_Y+1,MAX_X-WINDMETER_WIDTH,PANEL_Y+12,
					  hr.font8x8,
					  temp_colors,msg,STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
		gl_setclippingwindow(cx1,cy1,cx2,cy2);
	}
}


/* These funcs are for updating angle/power settings while dragging
 * sliders. Hmmm.*/
int change_angle()
{
	hr_update_mouse(NONE,BOB_REMOVE);
	update_control_panel(active_player,PANEL_ANGLE|PANEL_FREELY);
	hr_update_mouse(NONE,BOB_INIT);
	return 0;
}
int change_power()
{
	hr_update_mouse(NONE,BOB_REMOVE);
	update_control_panel(active_player,PANEL_POWER|PANEL_FREELY);
	hr_update_mouse(NONE,BOB_INIT);
	return 0;
}


/* Control handler ------------------------------------------*/
void control_handler(struct player *p)
{
	double angle2;
	int fire=0,x=0,ox,oy,l;
	struct hr_button *b;
	struct hr_zone *z;
	int (*func)();
	
	/* Delete all buttons and zones.*/
	while(hr_delete_button(hr.firstbutton));
	while(hr_delete_zone(hr.firstzone));
	/* Creates panels, buttons etc.*/
	create_control_panel();
	/* Total redraw.*/  
	update_control_panel(p,PANEL_NAME|PANEL_WIND|PANEL_TURN|PANEL_FIRE|
								PANEL_ANGLE|PANEL_POWER|PANEL_SHIELD|PANEL_WEAPON|
								PANEL_BARS);
	/* Random message.*/
	message(p->darkcolor,*(oneliners+random()%oneliner_n));
	
	/* AimAid(tm) bob sometimes overlaps with message area or windometer.
	 * --> potential gfx bug if message() is used.*/
	
	hr_clearkey();
	/* Loop until the dude FIREs! */
	hr_input_handler(NULL,NULL,AMB_RELEASED);
	while(!fire && !(game_flags & GAME_ABORTED)) {
		b=hr_input_handler(NULL,NULL,KBD_PRESSED|AMB_PRESSED);
		z=hr_get_zone(hr.firstzone,hr_mouse.x,hr_mouse.y,ZONE_ENABLED);
		
		/* Was a key pressed? */
		switch(hr_keyboard.lastkey){
		 case 27:
			/* Escape - Abort game! */
			if(hr_ask_confirmation("Abort game?"))
			  game_flags |= GAME_ABORTED;
			break;
		}
		
		/* Check if mouse pointer is positioned on game area */
		if(z==game_area) {			
			/* Aiming "intelligently"... */
			if(HR_LMB) {
				while(HR_LMB) {
					/* Mouse pointer must be above the gun.
					 * No gun can use angles <0 or >180. */
					ox=hr_mouse.x;
					oy=hr_mouse.y;
					hr_update_mouse(NONE,BOB_REMOVE);
					if((ox!=hr_mouse.x)||(oy!=hr_mouse.y)){
						if(hr_mouse.y<(p->y)) {
							angle2=hr_return_angle((double)(hr_mouse.x-(p->x)),
														  (double)((p->y)-hr_mouse.y));
							p->angle=(int)(RAD2DEG(angle2));
#ifndef WEIRD_KLUDGES_UNITE
							if(p->angle<0)
							  p->angle=0;
							else if(p->angle>180)
							  p->angle=180;
#endif
							update_control_panel(p,PANEL_ANGLE|PANEL_FREELY);
						}
					}
					hr_update_mouse(NONE,BOB_INIT);
				}
				update_control_panel(p,PANEL_ANGLE);
			}
			/* Opens an info window */
			if(HR_RMB)
			  info_window(hr_mouse.x,hr_mouse.y);
		}
		
		/* Set func point to null... */
		func=NULL;
		/* Buttons that respond immediately.*/
		if(b!=NULL) {
			for(l=0;l<MAX_CONTROLBUTTONS;l++){
				if(ctrlbutton[l]==b){
					switch(l){
					 case 2:
						/* Inc angle*/
						add_angle(p,1+((HR_RMB)?(4):(0)));
						update_control_panel(p,PANEL_ANGLE);
						break;
					 case 3:
						/* Change angle freely */
						ox=hr_mouse.x;
						oy=hr_mouse.y;
						update_control_panel(p,PANEL_ANGLE|PANEL_FREELY);
						while((HR_LMB)|(HR_RMB)) {
							x=hr_mouse.x;
							hr_update_mouse(WAIT_EVENT,BOB_NOIMG);
							if(x!=hr_mouse.x) {
								add_angle(p,x-hr_mouse.x);
								update_control_panel(p,PANEL_ANGLE|PANEL_FREELY);
							}
						}	
						mouse_setposition(ox,oy);
						break;
					 case 4:
						/* Dec angle */
						add_angle(p,-1-((HR_RMB)?(4):(0)));
						update_control_panel(p,PANEL_ANGLE);
						break;
					 case 5:
						/* Dec power */
						add_power(p,-1-((HR_RMB)?(4):(0)));
						update_control_panel(p,PANEL_POWER);
						break;
					 case 6:
						/* Change power freely */
						ox=hr_mouse.x;
						oy=hr_mouse.y;
						update_control_panel(p,PANEL_POWER|PANEL_FREELY);
						while((HR_LMB)||(HR_RMB))  {
							x=hr_mouse.x;
							hr_update_mouse(WAIT_EVENT,BOB_NOIMG);
							if(x!=hr_mouse.x)  {
								add_power(p,(hr_mouse.x-x)*2);
								update_control_panel(p,PANEL_POWER|PANEL_FREELY);
							}
						}
						mouse_setposition(ox,oy);
						break;
					 case 7:
						/* Inc power */
						add_power(p,1+((HR_RMB)?(4):(0)));
						update_control_panel(p,PANEL_POWER);
						break;
					 case 8:
						/* QuickAngle(tm) */
						func=&change_angle;
						break;
					 case 9:
						/* QuickoPower(tm) */
						func=&change_power;
						break;
					}
					break;
				}
			}
		}
		
		/* Wait until mouse keys are released.*/
		b=hr_input_handler(b,func,LMB_RELEASED|RMB_RELEASED);
		/* Buttons that respond after the mouse key is released.*/
		if(b!=NULL){
			for(l=0;l<MAX_CONTROLBUTTONS;l++){
				if(ctrlbutton[l]==b){
					switch(l) {
					 case 0:
						change_active_item(p,0);
						update_control_panel(p,PANEL_WEAPON|PANEL_SHIELD);
						break;
					 case 1:
						change_active_item(p,1);
						update_control_panel(p,PANEL_SHIELD|PANEL_WEAPON);
						break;
					 case 2:
					 case 3:
					 case 4:
					 case 8:
						/* Reset angle buttons */
						update_control_panel(p,PANEL_ANGLE);
						break;
					 case 5:
					 case 6:
					 case 7:
					 case 9:
						/* Reset power buttons */
						update_control_panel(p,PANEL_POWER);
						break;
					 case 10:
						/* FIRE. */
						fire=1;
						break;
					 case 11:
						control_colors();
						break;
#if 0
					 case 12:
						/* Settings */
						option_onoff(kuukkeliskuu);/*FIX*/
						break;
#endif
					}
					break;
				}
			}
		}
	}
	gl_setclippingwindow(0,0,MAX_X,MAX_Y);
	if(game_flags & AIM_AID)
	  hr_update_bob(crosshairs,p->x,p->y-20,NULL,BOB_REMOVE);
	gl_setclippingwindow(0,0,MAX_X,S_WIDTH-1);
	/* Delete all buttons and zones.*/
	while(hr_delete_button(hr.firstbutton));
	while(hr_delete_zone(hr.firstzone));
	/* Free button pointer table.*/
	free(ctrlbutton);
}
